from collections import defaultdict

from asyncapi_container.containers.v3.simple_spec import SimpleSpecV3

from django_asyncapi.settings import django_asyncapi_settings


class BusRoutingsMerger:
    """Merge a list of bus routing instances into one BusRouting instance."""

    def merge(self, asyncapi_specs: list[SimpleSpecV3]) -> SimpleSpecV3:
        first_spec = asyncapi_specs[0]

        merged_produces_routing_map = defaultdict(list)
        merged_consumes_routing_map = defaultdict(list)

        for asyncapi_spec in asyncapi_specs:
            for topic_name, messages in asyncapi_spec.sends.items():
                merged_produces_routing_map[topic_name].extend(messages)
                # deduplicate
                deduplicated_messages: set = set(merged_produces_routing_map[topic_name])
                deduplicated_messages: list = list(deduplicated_messages)
                merged_produces_routing_map[topic_name] = deduplicated_messages

            for topic_name, messages in asyncapi_spec.receives.items():
                merged_consumes_routing_map[topic_name].extend(messages)
                # deduplicate
                deduplicated_messages: set = set(merged_consumes_routing_map[topic_name])
                deduplicated_messages: list = list(deduplicated_messages)
                merged_consumes_routing_map[topic_name] = deduplicated_messages

        return SimpleSpecV3(
            info=first_spec.info,
            sends=merged_produces_routing_map,
            receives=merged_consumes_routing_map,
        )


def retrieve_asyncapi_spec_containers() -> list[SimpleSpecV3]:
    asyncapi_spec_classes = django_asyncapi_settings.ASYNCAPI_SPEC_CLASS
    if not isinstance(asyncapi_spec_classes, list):
        asyncapi_spec_classes = [asyncapi_spec_classes]

    asyncapi_spec_classes = [spec_class() for spec_class in asyncapi_spec_classes]

    return asyncapi_spec_classes


def retrieve_merged_asyncapi_container() -> SimpleSpecV3:
    asyncapi_spec_containers = retrieve_asyncapi_spec_containers()
    merger = BusRoutingsMerger()
    return merger.merge(asyncapi_specs=asyncapi_spec_containers)



from django.urls import path

from django_bus.views.asyncapi import AsyncAPIV3DocsView, AsyncAPIV3JsonView


urlpatterns = [
    path("asyncapi/v3/", AsyncAPIV3DocsView.as_view()),
    path("asyncapi/v3/json/", AsyncAPIV3JsonView.as_view()),
]



from asyncapi_container.asyncapi.generators.v3.generator import AsyncAPISpecV3Generator
from django.http import JsonResponse
from django.views.generic import TemplateView, View

from django_asyncapi.utils import retrieve_asyncapi_spec_containers, retrieve_merged_asyncapi_container


class AsyncAPIV3DocsView(TemplateView):
    template_name = "asyncapi.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        asyncapi_generator = AsyncAPISpecV3Generator(
            asyncapi_spec_container=retrieve_merged_asyncapi_container()
        )

        spec_containers = retrieve_asyncapi_spec_containers()
        first_spec_container = spec_containers[0]

        config = {
            "title": first_spec_container.info.title,
            "version": first_spec_container.info.version,
            "schema": asyncapi_generator.as_json(),
            "config": {
                "show": {
                    "sidebar": True,
                    "info": True,
                    "servers": True,
                    "operations": True,
                    "messages": True,
                    "schemas": True,
                    "errors": True,
                },
                "expand": {
                    "messageExamples": False,
                },
                "sidebar": {
                    "showServers": "byDefault",
                    "showOperations": "byDefault",
                },
            },
        }

        context["asyncapi_json"] = config

        context["service_title"] = first_spec_container.info.title
        return context


class AsyncAPIV3JsonView(View):
    def get(self, request, *args, **kwargs):
        asyncapi_generator = AsyncAPISpecV3Generator(
            asyncapi_spec_container=retrieve_merged_asyncapi_container()
        )
        asyncapi: dict = asyncapi_generator.as_dict()

        return JsonResponse(data=asyncapi, json_dumps_params={"ensure_ascii": False, "indent": 3})



from django.conf import settings
from pydantic import PyObject, BaseModel


class DjangoAsyncapiSettings(BaseModel):
    ASYNCAPI_SPEC_CLASS: list[PyObject] | PyObject



DJANGO_ASYNCAPI_SETTINGS = getattr(settings, "DJANGO_ASYNCAPI", {})
django_asyncapi_settings = DjangoAsyncapiSettings(**DJANGO_ASYNCAPI_SETTINGS)

